/**********************************************************************************/
/*   Sample for cache.c (SH7262)                                                  */
/**********************************************************************************/
#include "iodefine.h"                                   /* Peripheral Header File */
#pragma section Cache                                   /* Section Name is PCache */

void Cache_Init(void)                                   /* Cache Initialize       */
{
  CCNT.CCR1.LONG = 0x00000909;                          /* ICI=ICE=OCI=OCE=1      */
  if( CCNT.CCR1.LONG )  ;                               /* Dummy Read             */
}                                                       /* WT=0 (write back)      */

void ICache_Invalidate(void *p)
{
unsigned int tag = (unsigned int)p;                     /* Cast address to int    */
  *(unsigned int *)(0xF0000008 + (tag & 0x7F0)) = tag & 0x1FFFF800;
                                                        /* A=1 (Don't care way)   */
}                                                       /* V=0 (purge)            */

void ICache_All_Invalidate(void)
{
  CCNT.CCR1.LONG |= 0x00000800;                         /* ICI=1 (invalidate)     */
  if( CCNT.CCR1.LONG )  ;                               /* Dummy Read             */
}

void OCache_Purge(void *p)
{
unsigned int tag = (unsigned int)p;                     /* Cast address to int    */
  *(unsigned int *)(0xF0800008 + (tag & 0x7F0)) = tag & 0x1FFFF800;
                                                        /* A=1 (Don't care way)   */
}                                                       /* U=V=0 (purge)          */

void OCache_WriteBack(void *p)
{
unsigned int tag = (unsigned int)p;                     /* Cast address to int    */
  *(unsigned int *)(0xF0800008 + (tag & 0x7F0)) = tag & 0x1FFFF800 | 1;
                                                        /* A=1 (Don't care way)   */
}                                                       /* U=0,V=1 (writeback)    */

void OCache_All_Purge(void)
{
unsigned int entry;
  for( entry=0 ; entry<4*128*16 ; entry+=16 )
    *(unsigned int *)(0xF0800000 + entry) = 0;          /* A=0,U=V=0 (purge)      */
}